@@ -37,10 +37,11 @@ home = os.path.expanduser("~")
 DEFAULT_LVM_SNAPSIZE = '1G'
 DEFAULT_LVM_MOUNT_BASEDIR = '/var/lib/freezer'
 DEFAULT_LVM_SNAP_BASENAME = 'freezer_backup_snap'
+DEFAULT_FTP_PORT = 21
 DEFAULT_SSH_PORT = 22
 
 _DEFAULT_LOG_LEVELS = ['amqp=WARN', 'amqplib=WARN', 'boto=WARN',
-                       'qpid=WARN', 'stevedore=WARN', 'oslo_log=INFO',
+                       'stevedore=WARN', 'oslo_log=INFO',
                        'iso8601=WARN',
                        'requests.packages.urllib3.connectionpool=WARN',
                        'urllib3.connectionpool=WARN', 'websocket=WARN',
@@ -61,7 +62,7 @@ DEFAULT_PARAMS = {
     'max_segment_size': 33554432, 'lvm_srcvol': None,
     'download_limit': -1, 'hostname': None, 'remove_from_date': None,
     'restart_always_level': False, 'lvm_dirmount': None,
-    'dereference_symlink': None,
+    'rsync_block_size': 4096, 'dereference_symlink': None,
     'config': None, 'mysql_conf': False,
     'insecure': False, 'lvm_snapname': None,
     'lvm_snapperm': 'ro', 'snapshot': None,
@@ -69,20 +70,26 @@ DEFAULT_PARAMS = {
     'encrypt_pass_file': None, 'volume': None, 'proxy': None,
     'cinder_vol_id': '', 'cindernative_vol_id': '',
     'cinderbrick_vol_id': '',
+    'cinder_vol_name': '',
     'nova_inst_id': '', '__version__': FREEZER_VERSION,
+    'nova_inst_name': '',
     'remove_older_than': None, 'restore_from_date': None,
     'upload_limit': -1, 'always_level': False, 'version': None,
     'dry_run': False, 'lvm_snapsize': DEFAULT_LVM_SNAPSIZE,
     'restore_abs_path': None, 'log_file': None, 'log_level': "info",
     'mode': 'fs', 'action': 'backup', 'shadow': '', 'shadow_path': '',
     'windows_volume': '', 'command': None, 'metadata_out': None,
-    'storage': 'swift', 'ssh_key': '', 'ssh_username': '', 'ssh_host': '',
-    'ssh_port': DEFAULT_SSH_PORT, 'access_key': '', 'secret_key': '',
-    'endpoint': '', 'compression': 'gzip', 'overwrite': False,
+    'storage': 'swift', 'ssh_key': os.path.join(home, '.ssh/id_rsa'),
+    'ssh_username': '', 'ssh_password': '', 'ssh_host': '',
+    'ssh_port': DEFAULT_SSH_PORT,
+    'access_key': '', 'secret_key': '', 'endpoint': '',
+    'compression': 'gzip', 'overwrite': False,
     'incremental': None, 'consistency_check': False,
     'consistency_checksum': None, 'nova_restore_network': None,
     'cindernative_backup_id': None, 'sync': True, 'engine_name': 'tar',
-    'timeout': 120, 'project_id': None,
+    'timeout': 120, 'project_id': None, 'ftp_username': '',
+    'ftp_password': '', 'ftp_host': '', 'ftp_port': DEFAULT_FTP_PORT,
+    'ftp_keyfile': '', 'ftp_certfile': '',
 }
 
 _COMMON = [
@@ -119,7 +126,7 @@ _COMMON = [
                     "nova(OpenStack Instance). Default set to fs"),
     cfg.StrOpt('engine',
                short='e',
-               choices=['tar', 'rsync', 'nova', 'osbrick'],
+               choices=['tar', 'rsync', 'rsyncv2', 'nova', 'osbrick'],
                dest='engine_name',
                default=DEFAULT_PARAMS['engine_name'],
                help="Engine to be used for backup/restore. "
@@ -292,8 +299,13 @@ _COMMON = [
                default=DEFAULT_PARAMS['max_segment_size'],
                dest='max_segment_size',
                help="Set the maximum file chunk size in bytes to upload to "
-                    "swift Default 33554432 bytes (32MB)"
+                    "swift. Default 33554432 bytes (32MB)"
                ),
+    cfg.IntOpt('rsync-block-size',
+               default=DEFAULT_PARAMS['rsync_block_size'],
+               dest='rsync_block_size',
+               help="Set the data block size of used by rsync to "
+                    "generate signature. Default 4096 bytes (4K)."),
     cfg.StrOpt('restore-abs-path',
                dest='restore_abs_path',
                default=DEFAULT_PARAMS['restore_abs_path'],
@@ -369,6 +381,11 @@ _COMMON = [
                default=DEFAULT_PARAMS['cinder_vol_id'],
                help="Id of cinder volume for backup"
                ),
+    cfg.StrOpt('cinder-vol-name',
+               dest='cinder_vol_name',
+               default=DEFAULT_PARAMS['cinder_vol_name'],
+               help="Name of cinder volume for backup"
+               ),
     cfg.StrOpt('cinderbrick-vol-id',
                dest='cinderbrick_vol_id',
                default=DEFAULT_PARAMS['cinderbrick_vol_id'],
@@ -389,6 +406,12 @@ _COMMON = [
                default=DEFAULT_PARAMS['nova_inst_id'],
                help="Id of nova instance for backup"
                ),
+    cfg.StrOpt('nova-inst-name',
+               dest='nova_inst_name',
+               default=DEFAULT_PARAMS['nova_inst_name'],
+               help="Name of nova instance for backup"
+               ),
+
     cfg.StrOpt('project-id',
                dest='project_id',
                default=DEFAULT_PARAMS['project_id'],
@@ -415,12 +438,12 @@ _COMMON = [
     cfg.StrOpt('storage',
                dest='storage',
                default=DEFAULT_PARAMS['storage'],
-               choices=['local', 'swift', 'ssh', 's3'],
-               help="Storage for backups. Can be Swift, Local, SSH and S3 "
-                    "now. Swift is default storage now. Local stores backups"
-                    "on the same defined path, swift will store files in "
-                    "container, and s3 will store files in bucket in S3 "
-                    "compatible storage."
+               choices=['local', 'swift', 'ssh', 's3', 'ftp', 'ftps'],
+               help="Storage for backups. Can be Swift, Local, SSH(SFTP), "
+                    "FTP, FTPS and S3 now. Swift is default storage now. "
+                    "Local stores backups on the same defined path, "
+                    "swift will store files in container, and S3 will "
+                    "store files in bucket in S3 compatible storage."
                ),
     cfg.StrOpt('access-key',
                dest='access_key',
@@ -440,22 +463,28 @@ _COMMON = [
     cfg.StrOpt('ssh-key',
                dest='ssh_key',
                default=DEFAULT_PARAMS['ssh_key'],
-               help="Path to ssh-key for ssh storage only"
+               help="Path to ssh-key for ssh(sftp) storage only"
+               ),
+    cfg.StrOpt('ssh-password',
+               dest='ssh_password',
+               default=DEFAULT_PARAMS['ssh_password'],
+               help="Remote password for SSH(SFTP) storage"
                ),
     cfg.StrOpt('ssh-username',
                dest='ssh_username',
                default=DEFAULT_PARAMS['ssh_username'],
-               help="Remote username for ssh storage only"
+               help="Remote username for SSH(SFTP)) storage only"
                ),
     cfg.StrOpt('ssh-host',
                dest='ssh_host',
                default=DEFAULT_PARAMS['ssh_host'],
-               help="Remote host for ssh storage only"
+               help="Remote host for SSH(SFTP) storage only"
                ),
     cfg.IntOpt('ssh-port',
                dest='ssh_port',
                default=DEFAULT_PARAMS['ssh_port'],
-               help="Remote port for ssh storage only (default 22)"
+               help="Remote port for SSH(SFTP) storage"
+                    " only (default 22)"
                ),
     cfg.StrOpt('config',
                dest='config',
@@ -507,17 +536,54 @@ _COMMON = [
                     "this time it will raise a TimeOut Exception. Default is"
                     " {0}".format(DEFAULT_PARAMS['timeout'])
                ),
+    cfg.IntOpt('fullbackup-rotation',
+               dest='fullbackup_rotation',
+               default=1,
+               min=1,
+               help="Keep the last N fullbackups of cinder-volume, "
+                    "the parameter should be greater than 0. "
+                    "If set action to admin and set the parameter, "
+                    "it should keep the last N fullbackups, "
+                    "other backups should be deleted"),
+    cfg.StrOpt('ftp-password',
+               dest='ftp_password',
+               default=DEFAULT_PARAMS['ftp_password'],
+               help="Remote password for FTP, FTPS storage"
+               ),
+    cfg.StrOpt('ftp-username',
+               dest='ftp_username',
+               default=DEFAULT_PARAMS['ftp_username'],
+               help="Remote username for FTP, FTPS storage"
+               ),
+    cfg.StrOpt('ftp-host',
+               dest='ftp_host',
+               default=DEFAULT_PARAMS['ftp_host'],
+               help="Remote host for FTP, FTPS storage"
+               ),
+    cfg.IntOpt('ftp-port',
+               dest='ftp_port',
+               default=DEFAULT_PARAMS['ftp_port'],
+               help="Remote port for FTP, FTPS storage (default 21)"
+               ),
+    cfg.StrOpt('ftp-keyfile',
+               dest='ftp_keyfile',
+               default=DEFAULT_PARAMS['ftp_keyfile'],
+               help="Required if ftps server requires client certificate"
+               ),
+    cfg.StrOpt('ftp-certfile',
+               dest='ftp_certfile',
+               default=DEFAULT_PARAMS['ftp_certfile'],
+               help="Required if ftps server requires client certificate"
+               ),
 ]
 
 
 def config(args=[]):
     CONF.register_opts(_COMMON)
     CONF.register_cli_opts(_COMMON)
-    default_conf = None
     log.register_options(CONF)
     CONF(args=args,
          project='freezer',
-         default_config_files=default_conf,
          version=FREEZER_VERSION)
 
 
@@ -553,6 +619,7 @@ def get_backup_args():
         defaults.update(conf.default)
         for config_key in conf.default.keys():
             try:
+                CONF.get(config_key)
                 CONF.set_override(config_key, conf.default[config_key])
             except NoSuchOptError:
                 LOG.debug('No such opt, {0}, so set it'.format(config_key))
@@ -570,29 +637,24 @@ def get_backup_args():
 
     if not CONF.get('log_file'):
         log_file = None
-        for file_name in ['/var/log/freezer-agent/freezer-agent.log',
-                          '/var/log/freezer.log']:
-            try:
-                log_file = prepare_logging(file_name)
-            except IOError:
-                pass
+        file_name = '/var/log/freezer-agent/freezer-agent.log'
+        try:
+            log_file = prepare_logging(file_name)
+        except IOError:
+            pass
 
         if not log_file:
             # Set default working directory to ~/.freezer. If the directory
             # does not exists it is created
-            work_dir = os.path.join(home, '.freezer')
-            if not os.path.exists(work_dir):
-                try:
-                    os.makedirs(work_dir)
-                    log_file = prepare_logging(os.path.join(work_dir,
-                                                            'freezer.log'))
-                except (OSError, IOError) as err_msg:
-                    # This avoids freezer-agent to crash if it can't write to
-                    # ~/.freezer, which may happen on some env (for me,
-                    # it happens in Jenkins, as freezer-agent can't write to
-                    # /var/lib/jenkins).
-                    print(encodeutils.safe_decode('{}'.format(err_msg)),
-                          file=sys.stderr)
+            try:
+                log_file = prepare_logging()
+            except (OSError, IOError) as err_msg:
+                # This avoids freezer-agent to crash if it can't write to
+                # ~/.freezer, which may happen on some env (for me,
+                # it happens in Jenkins, as freezer-agent can't write to
+                # /var/lib/jenkins).
+                print(encodeutils.safe_decode('{}'.format(err_msg)),
+                      file=sys.stderr)
         if log_file:
             CONF.set_default('log_file', log_file)
         else:
@@ -644,12 +706,13 @@ def get_backup_args():
                 backup_args.path_to_backup[:3]
 
     backup_media = 'fs'
-    if backup_args.cinder_vol_id:
+    if backup_args.cinder_vol_id or backup_args.cinder_vol_name:
         backup_media = 'cinder'
     elif backup_args.cindernative_vol_id or backup_args.cindernative_backup_id:
         backup_media = 'cindernative'
     elif backup_args.engine_name == 'nova' and (backup_args.project_id or
-                                                backup_args.nova_inst_id):
+                                                backup_args.nova_inst_id or
+                                                backup_args.nova_inst_name):
         backup_media = 'nova'
     elif backup_args.cinderbrick_vol_id:
         backup_media = 'cinderbrick'
