@@ -1,24 +1,21 @@
-"""
-(c) Copyright 2014,2015 Hewlett-Packard Development Company, L.P.
+# (c) Copyright 2014,2015 Hewlett-Packard Development Company, L.P.
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#     http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
 
-Licensed under the Apache License, Version 2.0 (the "License");
-you may not use this file except in compliance with the License.
-You may obtain a copy of the License at
-
-    http://www.apache.org/licenses/LICENSE-2.0
-
-Unless required by applicable law or agreed to in writing, software
-distributed under the License is distributed on an "AS IS" BASIS,
-WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-See the License for the specific language governing permissions and
-limitations under the License.
-
-"""
 
 import abc
 import json
 import multiprocessing
-from multiprocessing import queues
 import shutil
 import tempfile
 import time
@@ -33,6 +30,12 @@ from freezer.storage import base
 from freezer.utils import streaming
 from freezer.utils import utils
 
+if six.PY3:
+    from multiprocessing import SimpleQueue
+else:
+    from multiprocessing.queues import SimpleQueue
+
+
 LOG = log.getLogger(__name__)
 
 
@@ -50,24 +53,28 @@ class BackupEngine(object):
     This class is an abstraction over all implementations.
 
     Workflow:
+
     1) invoke backup
-        1.1) try to download metadata for incremental
-        1.2) create a dataflow between backup_stream and storage.write_backup
-            Backup_stream is producer of data, for tar backup
-            it creates a gnutar subprocess and start to read data from stdout
-            Storage write_backup is consumer of data, it creates a thread
-            that store data in storage.
-            Both streams communicate in non-blocking mode
-        1.3) invoke post_backup - now it uploads metadata file
+
+       1) try to download metadata for incremental
+       2) create a dataflow between backup_stream and storage.write_backup
+          Backup_stream is producer of data, for tar backup
+          it creates a gnutar subprocess and start to read data from stdout
+          Storage write_backup is consumer of data, it creates a thread
+          that store data in storage.
+          Both streams communicate in non-blocking mode
+       3) invoke post_backup - now it uploads metadata file
+
     2) restore backup
-        2.1) define all incremental backups
-        2.2) for each incremental backup create a dataflow between
-            storage.read_backup and restore_stream
-            Read_backup is data producer, it reads data chunk by chunk from
-            the specified storage and pushes the chunks into a queue.
-            Restore stream is a consumer, that is actually does restore (for
-            tar it is a thread that creates gnutar subprocess and feeds chunks
-            to stdin of this thread.
+
+       1) define all incremental backups
+       2) for each incremental backup create a dataflow between
+          storage.read_backup and restore_stream
+          Read_backup is data producer, it reads data chunk by chunk from
+          the specified storage and pushes the chunks into a queue.
+          Restore stream is a consumer, that is actually does restore (for
+          tar it is a thread that creates gnutar subprocess and feeds chunks
+          to stdin of this thread.
 
     :type storage: freezer.storage.base.Storage
     """
@@ -88,7 +95,7 @@ class BackupEngine(object):
         """
         pass
 
-    def backup_stream(self, backup_path, rich_queue, manifest_path):
+    def backup_stream(self, backup_resource, rich_queue, manifest_path):
         """
         :param rich_queue:
         :type rich_queue: freezer.streaming.RichQueue
@@ -96,9 +103,10 @@ class BackupEngine(object):
         :param manifest_path:
         :return:
         """
-        rich_queue.put_messages(self.backup_data(backup_path, manifest_path))
+        rich_queue.put_messages(self.backup_data(backup_resource,
+                                                 manifest_path))
 
-    def backup(self, backup_path, hostname_backup_name, no_incremental,
+    def backup(self, backup_resource, hostname_backup_name, no_incremental,
                max_level, always_level, restart_always_level, queue_size=2):
         """
         Here we now location of all interesting artifacts like metadata
@@ -145,7 +153,7 @@ class BackupEngine(object):
                 self.backup_stream,
                 input_queue,
                 read_except_queue,
-                kwargs={"backup_path": backup_path,
+                kwargs={"backup_resource": backup_resource,
                         "manifest_path": engine_meta})
 
             write_stream = streaming.QueuedThread(
@@ -182,7 +190,7 @@ class BackupEngine(object):
                     "Engine error. Failed to backup.")
 
             with open(freezer_meta, mode='wb') as b_file:
-                b_file.write(json.dumps(self.metadata()))
+                b_file.write(json.dumps(self.metadata(backup_resource)))
             self.storage.put_metadata(engine_meta, freezer_meta, backup)
         finally:
             shutil.rmtree(tmpdir)
@@ -214,9 +222,10 @@ class BackupEngine(object):
             except_queue.put(e)
             raise
 
-    def restore(self, hostname_backup_name, restore_path,
+    def restore(self, hostname_backup_name, restore_resource,
                 overwrite,
-                recent_to_date):
+                recent_to_date,
+                backup_media=None):
         """
 
         :param hostname_backup_name:
@@ -224,16 +233,19 @@ class BackupEngine(object):
         :param overwrite:
         :param recent_to_date:
         """
-        LOG.info("Creating restore path: {0}".format(restore_path))
-        # if restore path can't be created this function will raise exception
-        utils.create_dir_tree(restore_path)
-        if not overwrite and not utils.is_empty_dir(restore_path):
-            raise Exception(
-                "Restore dir is not empty. "
-                "Please use --overwrite or provide different path "
-                "or remove the content of {}".format(restore_path))
+        if backup_media == 'fs':
+            LOG.info("Creating restore path: {0}".format(restore_resource))
+            # if restore path can't be created this function will raise
+            # exception
+            utils.create_dir_tree(restore_resource)
+            if not overwrite and not utils.is_empty_dir(restore_resource):
+                raise Exception(
+                    "Restore dir is not empty. "
+                    "Please use --overwrite or provide different path "
+                    "or remove the content of {}".format(restore_resource))
+
+            LOG.info("Restore path creation completed")
 
-        LOG.info("Restore path creation completed")
         backups = self.storage.get_latest_level_zero_increments(
             engine=self,
             hostname_backup_name=hostname_backup_name,
@@ -242,7 +254,7 @@ class BackupEngine(object):
         max_level = max(backups.keys())
 
         # Use SimpleQueue because Queue does not work on Mac OS X.
-        read_except_queue = queues.SimpleQueue()
+        read_except_queue = SimpleQueue()
         LOG.info("Restoring backup {0}".format(hostname_backup_name))
         for level in range(0, max_level + 1):
             LOG.info("Restoring from level {0}".format(level))
@@ -259,11 +271,11 @@ class BackupEngine(object):
             # Start the tar pipe consumer process
 
             # Use SimpleQueue because Queue does not work on Mac OS X.
-            write_except_queue = queues.SimpleQueue()
+            write_except_queue = SimpleQueue()
 
             engine_stream = multiprocessing.Process(
                 target=self.restore_level,
-                args=(restore_path, read_pipe, backup, write_except_queue))
+                args=(restore_resource, read_pipe, backup, write_except_queue))
 
             engine_stream.daemon = True
             engine_stream.start()
@@ -311,5 +323,5 @@ class BackupEngine(object):
         pass
 
     @abc.abstractmethod
-    def metadata(self):
+    def metadata(self, backup_resource):
         pass
