@@ -1,16 +1,18 @@
-"""Freezer rsync incremental engine
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#     http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
 
-Licensed under the Apache License, Version 2.0 (the "License");
-you may not use this file except in compliance with the License.
-You may obtain a copy of the License at
 
-    http://www.apache.org/licenses/LICENSE-2.0
-
-Unless required by applicable law or agreed to in writing, software
-distributed under the License is distributed on an "AS IS" BASIS,
-WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-See the License for the specific language governing permissions and
-limitations under the License.
+"""
+Freezer rsync incremental engine
 """
 
 import getpass
@@ -18,7 +20,6 @@ import grp
 import json
 import os
 import pwd
-import Queue
 import re
 import stat
 import sys
@@ -26,6 +27,7 @@ import threading
 
 from oslo_log import log
 from six.moves import cStringIO
+from six.moves import queue
 
 from freezer.engine import engine
 from freezer.engine.rsync import pyrsync
@@ -101,7 +103,7 @@ class RsyncEngine(engine.BackupEngine):
             self.cipher = crypt.AESEncrypt(self.encrypt_pass_file)
             data_chunk += self.cipher.generate_header()
 
-        rsync_queue = Queue.Queue(maxsize=2)
+        rsync_queue = queue.Queue(maxsize=2)
 
         t_get_sign_delta = threading.Thread(
             target=self.get_sign_delta,
@@ -246,7 +248,7 @@ class RsyncEngine(engine.BackupEngine):
         """
 
         if not old_file_meta:
-            raise StopIteration
+            return
 
         # If the ctime or mtime has changed, the delta is computed
         # data block is returned
