@@ -1,22 +1,21 @@
-"""
-(c) Copyright 2014,2015 Hewlett-Packard Development Company, L.P.
-(C) Copyright 2016 Hewlett Packard Enterprise Development Company LP
+# (c) Copyright 2014,2015 Hewlett-Packard Development Company, L.P.
+# (C) Copyright 2016 Hewlett Packard Enterprise Development Company LP
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#     http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
 
-Licensed under the Apache License, Version 2.0 (the "License");
-you may not use this file except in compliance with the License.
-You may obtain a copy of the License at
-
-    http://www.apache.org/licenses/LICENSE-2.0
-
-Unless required by applicable law or agreed to in writing, software
-distributed under the License is distributed on an "AS IS" BASIS,
-WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-See the License for the specific language governing permissions and
-limitations under the License.
-
-"""
 
 import abc
+from concurrent import futures
 import datetime
 import os
 import sys
@@ -27,6 +26,8 @@ from oslo_log import log
 from oslo_utils import importutils
 import six
 
+from freezer.common import client_manager
+from freezer.openstack import admin
 from freezer.openstack import backup
 from freezer.openstack import restore
 from freezer.snapshot import snapshot
@@ -49,8 +50,21 @@ class Job(object):
         self.conf = conf_dict
         self.storage = storage
         self.engine = conf_dict.engine
+        self.client = client_manager.get_client_manager(CONF)
+        self.nova = self.client.get_nova()
+        self.cinder = self.client.get_cinder()
         self._general_validation()
         self._validate()
+        if self.conf.nova_inst_name:
+            self.nova_instance_ids = [server.id for server in
+                                      self.nova.servers.list(detailed=False)
+                                      if server.name ==
+                                      self.conf.nova_inst_name]
+        if self.conf.cinder_vol_name:
+            self.cinder_vol_ids = [volume.id for volume in
+                                   self.cinder.volumes.list()
+                                   if volume.name ==
+                                   self.conf.cinder_inst_name]
 
     @abc.abstractmethod
     def _validate(self):
@@ -93,12 +107,20 @@ class InfoJob(Job):
         fields = ["Container", "Size", "Object Count"]
         data = []
         for container in info:
+            if self.conf.container:
+                container_name = container.get('container_name')
+                if container_name != self.conf.container:
+                    continue
+
             values = [
                 container.get('container_name'),
                 container.get('size'),
                 container.get('objects_count')
             ]
             data.append(values)
+
+            if self.conf.container:
+                break  # values for given container were found
         return [fields, data]
 
 
@@ -117,13 +139,15 @@ class BackupJob(Job):
             if not self.conf.no_incremental:
                 raise ValueError("Incremental nova backup is not supported")
 
-            if not self.conf.nova_inst_id and not self.conf.project_id:
-                raise ValueError("nova-inst-id or project-id"
+            if not self.conf.nova_inst_id and not self.conf.project_id \
+                    and not self.conf.nova_inst_name:
+                raise ValueError("nova-inst-id or project-id or nova-inst-name"
                                  " argument must be provided")
 
         if self.conf.mode == 'cinder':
-            if not self.conf.cinder_vol_id:
-                raise ValueError("cinder-vol-id argument must be provided")
+            if not self.conf.cinder_vol_id and not self.conf.cinder_vol_name:
+                raise ValueError("cinder-vol-id or cinder-vol-name argument "
+                                 "must be provided")
 
         if self.conf.mode == "cindernative":
             if not self.conf.cindernative_vol_id:
@@ -145,8 +169,7 @@ class BackupJob(Job):
                     LOG.error('Error while sync exec: {0}'.format(err))
         except Exception as error:
             LOG.error('Error while sync exec: {0}'.format(error))
-        if not self.conf.mode:
-            raise ValueError("Empty mode")
+
         mod_name = 'freezer.mode.{0}.{1}'.format(
             self.conf.mode, self.conf.mode.capitalize() + 'Mode')
         app_mode = importutils.import_object(mod_name, self.conf)
@@ -177,7 +200,6 @@ class BackupJob(Job):
                   'log_file',
                   'storage',
                   'mode',
-                  'os_auth_version',
                   'proxy',
                   'compression',
                   'ssh_key',
@@ -260,9 +282,11 @@ class BackupJob(Job):
                     max_level=self.conf.max_level,
                     always_level=self.conf.always_level,
                     restart_always_level=self.conf.restart_always_level)
-            else:
+
+            elif self.conf.nova_inst_id:
                 LOG.info('Executing nova backup. Instance ID: {0}'.format(
                     self.conf.nova_inst_id))
+
                 hostname_backup_name = os.path.join(
                     self.conf.hostname_backup_name,
                     self.conf.nova_inst_id)
@@ -274,6 +298,25 @@ class BackupJob(Job):
                     always_level=self.conf.always_level,
                     restart_always_level=self.conf.restart_always_level)
 
+            else:
+                executor = futures.ThreadPoolExecutor(
+                    max_workers=len(self.nova_instance_ids))
+                futures_list = []
+                for instance_id in self.nova_instance_ids:
+                    hostname_backup_name = os.path.join(
+                        self.conf.hostname_backup_name, instance_id)
+                    futures_list.append(executor.submit(
+                        self.engine.backup(
+                            backup_resource=instance_id,
+                            hostname_backup_name=hostname_backup_name,
+                            no_incremental=self.conf.no_incremental,
+                            max_level=self.conf.max_level,
+                            always_level=self.conf.always_level,
+                            restart_always_level=self.conf.restart_always_level
+                        )))
+
+                futures.wait(futures_list, CONF.timeout)
+
         elif backup_media == 'cindernative':
             LOG.info('Executing cinder native backup. Volume ID: {0}, '
                      'incremental: {1}'.format(self.conf.cindernative_vol_id,
@@ -282,9 +325,22 @@ class BackupJob(Job):
                                     name=self.conf.backup_name,
                                     incremental=self.conf.incremental)
         elif backup_media == 'cinder':
-            LOG.info('Executing cinder snapshot. Volume ID: {0}'.format(
-                self.conf.cinder_vol_id))
-            backup_os.backup_cinder_by_glance(self.conf.cinder_vol_id)
+            if self.conf.cinder_vol_id:
+                LOG.info('Executing cinder snapshot. Volume ID: {0}'.format(
+                    self.conf.cinder_vol_id))
+                backup_os.backup_cinder_by_glance(self.conf.cinder_vol_id)
+            else:
+                executor = futures.ThreadPoolExecutor(
+                    max_workers=len(self.cinder_vol_ids))
+                futures_list = []
+                for instance_id in self.cinder_vol_ids:
+                    LOG.info('Executing cinder snapshot. Volume ID:'
+                             ' {0}'.format(instance_id))
+                    futures_list.append(executor.submit(
+                        backup_os.backup_cinder_by_glance(instance_id)))
+
+                futures.wait(futures_list, CONF.timeout)
+
         elif backup_media == 'cinderbrick':
             LOG.info('Executing cinder volume backup using os-brick. '
                      'Volume ID: {0}'.format(self.conf.cinderbrick_vol_id))
@@ -305,7 +361,9 @@ class RestoreJob(Job):
     def _validate(self):
         if not any([self.conf.restore_abs_path,
                     self.conf.nova_inst_id,
+                    self.conf.nova_inst_name,
                     self.conf.cinder_vol_id,
+                    self.conf.cinder_vol_name,
                     self.conf.cindernative_vol_id,
                     self.conf.cinderbrick_vol_id,
                     self.conf.project_id]):
@@ -360,7 +418,8 @@ class RestoreJob(Job):
                     hostname_backup_name=self.conf.hostname_backup_name,
                     overwrite=conf.overwrite,
                     recent_to_date=restore_timestamp)
-            else:
+
+            elif conf.nova_inst_id:
                 LOG.info("Restoring nova backup. Instance ID: {0}, "
                          "timestamp: {1} network-id {2}".format(
                              conf.nova_inst_id,
@@ -376,11 +435,36 @@ class RestoreJob(Job):
                     recent_to_date=restore_timestamp,
                     backup_media=conf.mode)
 
+            else:
+                for instance_id in self.nova_instance_ids:
+                    LOG.info("Restoring nova backup. Instance ID: {0}, "
+                             "timestamp: {1} network-id {2}".format(
+                                 instance_id, restore_timestamp,
+                                 conf.nova_restore_network))
+                    hostname_backup_name = os.path.join(
+                        self.conf.hostname_backup_name, instance_id)
+                    self.engine.restore(
+                        hostname_backup_name=hostname_backup_name,
+                        restore_resource=instance_id,
+                        overwrite=conf.overwrite,
+                        recent_to_date=restore_timestamp,
+                        backup_media=conf.mode)
+
         elif conf.backup_media == 'cinder':
-            LOG.info("Restoring cinder backup from glance. Volume ID: {0}, "
-                     "timestamp: {1}".format(conf.cinder_vol_id,
-                                             restore_timestamp))
-            res.restore_cinder_by_glance(conf.cinder_vol_id, restore_timestamp)
+            if conf.cinder_vol_id:
+                LOG.info("Restoring cinder backup from glance. "
+                         "Volume ID: {0}, timestamp: {1}".format(
+                             conf.cinder_vol_id,
+                             restore_timestamp))
+                res.restore_cinder_by_glance(conf.cinder_vol_id,
+                                             restore_timestamp)
+            else:
+                for instance_id in self.cinder_vol_ids:
+                    LOG.info("Restoring cinder backup from glance. "
+                             "Volume ID: {0}, timestamp: {1}".format(
+                                 instance_id, restore_timestamp))
+                    res.restore_cinder_by_glance(instance_id,
+                                                 restore_timestamp)
         elif conf.backup_media == 'cindernative':
             LOG.info("Restoring cinder native backup. Volume ID {0}, Backup ID"
                      " {1}, timestamp: {2}".format(conf.cindernative_vol_id,
@@ -408,12 +492,24 @@ class AdminJob(Job):
 
     def _validate(self):
         # no validation required in this job
-        if not self.conf.remove_from_date and not self.conf.remove_older_than:
+        if self.conf.backup_media == 'cindernative':
+            if not self.conf.fullbackup_rotation:
+                raise Exception("The parameter --fullbackup-rotation "
+                                "is required")
+        elif not (self.conf.remove_from_date or self.conf.remove_older_than):
             raise ValueError("You need to provide to remove backup older "
                              "than this time. You can use --remove-older-than "
                              "or --remove-from-date")
 
     def execute(self):
+        # remove backups by freezer admin action
+        backup_media = self.conf.backup_media
+        if backup_media == 'cindernative':
+            admin_os = admin.AdminOs(self.conf.client_manager)
+            admin_os.del_off_limit_fullbackup(
+                self.conf.cindernative_vol_id,
+                self.conf.fullbackup_rotation)
+            return {}
         if self.conf.remove_from_date:
             timestamp = utils.date_to_timestamp(self.conf.remove_from_date)
         else:
@@ -422,13 +518,24 @@ class AdminJob(Job):
             timestamp = int(time.mktime(timestamp.timetuple()))
 
         if self.conf.backup_media == 'cinder':
-            old_backups = self.get_cinder_old_backups(
-                timestamp,
-                self.conf.cinder_vol_id
-            )
-            self.remove_backup_dirs(old_backups, self.conf.cinder_vol_id)
-            return {}
+            if self.conf.cinder_vol_id:
+                old_backups = self.get_cinder_old_backups(
+                    timestamp,
+                    self.conf.cinder_vol_id
+                )
+                self.remove_backup_dirs(old_backups,
+                                        self.conf.cinder_vol_id)
+                return {}
 
+            else:
+                for instance_id in self.cinder_vol_ids:
+                    old_backups = self.get_cinder_old_backups(
+                        timestamp,
+                        instance_id
+                    )
+                    self.remove_backup_dirs(old_backups,
+                                            instance_id)
+                    return {}
         hostname_backup_name_set = set()
 
         if self.conf.backup_media == 'nova':
@@ -439,11 +546,19 @@ class AdminJob(Job):
                     hostname_backup_name = os.path.join(
                         self.conf.hostname_backup_name, instance_id)
                     hostname_backup_name_set.add(hostname_backup_name)
-            else:
+
+            elif self.conf.nova_inst_id:
                 hostname_backup_name = os.path.join(
                     self.conf.hostname_backup_name,
                     self.conf.nova_inst_id)
                 hostname_backup_name_set.add(hostname_backup_name)
+
+            else:
+                for instance_id in self.nova_instance_ids:
+                    hostname_backup_name = os.path.join(
+                        self.conf.hostname_backup_name,
+                        instance_id)
+                    hostname_backup_name_set.add(hostname_backup_name)
         else:
             hostname_backup_name_set.add(self.conf.hostname_backup_name)
 
@@ -478,7 +593,8 @@ class AdminJob(Job):
                 self.storage.segments,
                 cinder_vol_id
             )
-        elif self.storage.type in ['local', 'ssh', 's3']:
+        elif self.storage.type in \
+                ['local', 'ssh', 's3', 'ftp', 'ftps']:
             path_prefix = "{0}/{1}".format(
                 self.storage.storage_path,
                 cinder_vol_id
