@@ -1,19 +1,20 @@
+# Copyright 2015 Hewlett-Packard
+# (c) Copyright 2016 Hewlett Packard Enterprise Development Company LP
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#     http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
+
+
 """
-Copyright 2015 Hewlett-Packard
-(c) Copyright 2016 Hewlett Packard Enterprise Development Company LP
-
-Licensed under the Apache License, Version 2.0 (the "License");
-you may not use this file except in compliance with the License.
-You may obtain a copy of the License at
-
-    http://www.apache.org/licenses/LICENSE-2.0
-
-Unless required by applicable law or agreed to in writing, software
-distributed under the License is distributed on an "AS IS" BASIS,
-WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-See the License for the specific language governing permissions and
-limitations under the License.
-
 Freezer main execution function
 """
 
@@ -25,16 +26,17 @@ import sys
 
 from oslo_config import cfg
 from oslo_log import log
+from oslo_utils import importutils
 
+from freezer.common import client_manager
 from freezer.common import config as freezer_config
 from freezer.engine import manager as engine_manager
 from freezer import job
-from freezer.openstack import osclients
 from freezer.storage import local
 from freezer.storage import multiple
+from freezer.storage import s3
 from freezer.storage import ssh
 from freezer.storage import swift
-from freezer.utils import config
 from freezer.utils import utils
 
 CONF = cfg.CONF
@@ -58,11 +60,33 @@ def freezer_main(backup_args):
     max_segment_size = backup_args.max_segment_size
     if (backup_args.storage ==
             'swift' or
-            backup_args.backup_media in ['nova', 'cinder', 'cindernative']):
+            backup_args.backup_media in ['nova', 'cinder', 'cindernative',
+                                         'cinderbrick']):
 
-        backup_args.client_manager = get_client_manager(backup_args.__dict__)
+        backup_args.client_manager = client_manager.get_client_manager(
+            backup_args.__dict__)
+
+    if backup_args.storage == 's3':
+        if backup_args.__dict__['access_key'] == '' \
+                and 'ACCESS_KEY' in os.environ:
+            backup_args.__dict__['access_key'] = os.environ.get('ACCESS_KEY')
+        if backup_args.__dict__['access_key'] == '':
+            raise Exception('No access key found for S3 compatible storage')
+
+        if backup_args.__dict__['secret_key'] == '' \
+                and 'SECRET_KEY' in os.environ:
+            backup_args.__dict__['secret_key'] = os.environ.get('SECRET_KEY')
+        if backup_args.__dict__['secret_key'] == '':
+            raise Exception('No secret key found for S3 compatible storage')
+
+        if backup_args.__dict__['endpoint'] == '' \
+                and 'ENDPOINT' in os.environ:
+            backup_args.__dict__['endpoint'] = os.environ.get('ENDPOINT')
+        if backup_args.__dict__['endpoint'] == '':
+            raise Exception('No endpoint found for S3 compatible storage')
 
     if backup_args.storages:
+        # pylint: disable=abstract-class-instantiated
         storage = multiple.MultipleStorage(
             [storage_from_dict(x, max_segment_size)
              for x in backup_args.storages])
@@ -76,6 +100,7 @@ def freezer_main(backup_args):
         exclude=backup_args.exclude,
         storage=storage,
         max_segment_size=backup_args.max_segment_size,
+        rsync_block_size=backup_args.rsync_block_size,
         encrypt_key=backup_args.encrypt_pass_file,
         dry_run=backup_args.dry_run
     )
@@ -129,7 +154,7 @@ def run_job(conf, storage):
     LOG.info('Job time Elapsed: {0}'.format(end_time - start_time))
     LOG.info('Backup metadata received: {0}'.format(json.dumps(response)))
     if not conf.quiet:
-        LOG.info("End freezer agent process successfully")
+        LOG.info("Run freezer agent process successfully")
 
     if conf.metadata_out and response:
         if conf.metadata_out == '-':
@@ -138,7 +163,7 @@ def run_job(conf, storage):
         else:
             with open(conf.metadata_out, 'w') as outfile:
                 outfile.write(json.dumps(response))
-    elif response:
+    elif response and isinstance(response, dict):
         pp = prettytable.PrettyTable(["Property", "Value"])
         for k, v in response.items():
             k = k.replace("_", " ")
@@ -146,6 +171,12 @@ def run_job(conf, storage):
         sys.stdout.writelines(pp.get_string())
         sys.stdout.write('\n')
         sys.stdout.flush()
+    elif response and isinstance(response, list):
+        pp = prettytable.PrettyTable()
+        pp.field_names = response[0]
+        for i in response[1]:
+            pp.add_row(i)
+        print(pp)
     else:
         return
 
@@ -161,27 +192,6 @@ def fail(exit_code, e, quiet, do_log=True):
     return exit_code
 
 
-def parse_osrc(file_name):
-    with open(file_name, 'r') as osrc_file:
-        return config.osrc_parse(osrc_file.read())
-
-
-def get_client_manager(backup_args):
-    if "osrc" in backup_args:
-        options = osclients.OpenstackOpts.create_from_dict(
-            parse_osrc(backup_args['osrc']))
-    else:
-        options = osclients.OpenstackOpts.create_from_env().get_opts_dicts()
-
-    client_manager = osclients.OSClientManager(
-        auth_url=options.pop('auth_url', None),
-        auth_method=options.pop('auth_method', 'password'),
-        dry_run=backup_args.get('dry_run', None),
-        **options
-    )
-    return client_manager
-
-
 def storage_from_dict(backup_args, max_segment_size):
     storage_name = backup_args['storage']
     container = backup_args['container']
@@ -191,17 +201,47 @@ def storage_from_dict(backup_args, max_segment_size):
 
         storage = swift.SwiftStorage(
             client_manager, container, max_segment_size)
+    elif storage_name == "s3":
+        storage = s3.S3Storage(
+            backup_args['access_key'],
+            backup_args['secret_key'],
+            backup_args['endpoint'],
+            container,
+            max_segment_size
+        )
     elif storage_name == "local":
         storage = local.LocalStorage(
             storage_path=container,
             max_segment_size=max_segment_size)
     elif storage_name == "ssh":
-        storage = ssh.SshStorage(
-            container,
-            backup_args['ssh_key'], backup_args['ssh_username'],
-            backup_args['ssh_host'],
-            int(backup_args.get('ssh_port', freezer_config.DEFAULT_SSH_PORT)),
-            max_segment_size=max_segment_size)
+        if backup_args['ssh_password']:
+            storage = ssh.SshStorage(
+                container,
+                backup_args['ssh_username'],
+                backup_args['ssh_host'],
+                int(backup_args['ssh_port']),
+                max_segment_size=max_segment_size,
+                remote_pwd=backup_args['ssh_password'])
+        else:
+            storage = ssh.SshStorage(
+                container,
+                backup_args['ssh_username'],
+                backup_args['ssh_host'],
+                int(backup_args['ssh_port']),
+                max_segment_size=max_segment_size,
+                ssh_key_path=backup_args['ssh_key'])
+    elif storage_name in ["ftp", "ftps"]:
+        args = [container, backup_args['ftp_password'],
+                backup_args['ftp_username'],
+                backup_args['ftp_host'], int(backup_args['ftp_port']),
+                max_segment_size]
+        if storage_name == 'ftps':
+            args.append(backup_args['ftp_keyfile'])
+            args.append(backup_args['ftp_certfile'])
+        LOG.info('args=%s' % args)
+        storage = importutils.import_object(
+            "freezer.storage.ftp.{0}Storage".format(
+                storage_name.capitalize()), *args)
     else:
         raise Exception("No storage found for name {0}".format(
             backup_args['storage']))
@@ -233,7 +273,7 @@ def main():
     except Exception as err:
         quiet = backup_args.quiet if backup_args else False
         LOG.exception(err)
-        LOG.critical("End freezer agent process unsuccessfully")
+        LOG.critical("Run freezer agent process unsuccessfully")
         return fail(1, err, quiet)
 
 if __name__ == '__main__':
