@@ -1,23 +1,26 @@
+# (c) Copyright 2014,2015 Hewlett-Packard Development Company, L.P.
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#     http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
+
+
 """
-(c) Copyright 2014,2015 Hewlett-Packard Development Company, L.P.
-
-Licensed under the Apache License, Version 2.0 (the "License");
-you may not use this file except in compliance with the License.
-You may obtain a copy of the License at
-
-    http://www.apache.org/licenses/LICENSE-2.0
-
-Unless required by applicable law or agreed to in writing, software
-distributed under the License is distributed on an "AS IS" BASIS,
-WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-See the License for the specific language governing permissions and
-limitations under the License.
-
 Freezer restore modes related functions
 """
 
 import json
 import os
+import shutil
+import tempfile
 import time
 
 from oslo_config import cfg
@@ -61,12 +64,13 @@ class RestoreOs(object):
         elif self.storage.type == "local":
             path = "{0}/{1}".format(self.container, path)
             backups = os.listdir(os.path.abspath(path))
-        elif self.storage.type == "ssh":
+        elif self.storage.type in ['ssh', 'ftp', 'ftps']:
             path = "{0}/{1}".format(self.container, path)
             backups = self.storage.listdir(path)
         else:
             msg = ("{} storage type is not supported at the moment."
-                   " Try local, swift or ssh".format(self.storage.type))
+                   " Try local, SWIFT, SSH(SFTP), FTP or FTPS ".
+                   format(self.storage.type))
             print(msg)
             raise BaseException(msg)
         backups = list(filter(lambda x: x >= restore_from_timestamp, backups))
@@ -152,7 +156,7 @@ class RestoreOs(object):
                 msg = "Failed to open image file {}".format(image_file)
                 LOG.error(msg)
                 raise BaseException(msg)
-            info = json.load(file(metadata_file))
+            info = json.load(open(metadata_file, 'r'))
             image = self.client_manager.create_image(
                 name="restore_{}".format(path),
                 container_format="bare",
@@ -177,6 +181,32 @@ class RestoreOs(object):
                 disk_format="raw",
                 data=data)
             return info, image
+        elif self.storage.type in ['ftp', 'ftps']:
+            image_file = "{0}/{1}/{2}/{3}".format(self.container, path,
+                                                  backup, path)
+            metadata_file = "{0}/{1}/{2}/metadata".format(self.container,
+                                                          path, backup)
+            try:
+                tmpdir = tempfile.mkdtemp()
+            except Exception:
+                LOG.error("Unable to create a tmp directory")
+                raise
+            try:
+                data_image = utils.path_join(tmpdir, "data_image")
+                LOG.info('create image restore ftp storage')
+                self.storage.get_file(image_file, data_image)
+                data_meta = utils.path_join(tmpdir, "data_meta")
+                self.storage.get_file(metadata_file, data_meta)
+                data = open(data_image, 'rb')
+                info = json.load(open(data_meta, 'r'))
+                image = self.client_manager.create_image(
+                    name="restore_{}".format(path),
+                    container_format="bare",
+                    disk_format="raw",
+                    data=data)
+                return info, image
+            finally:
+                shutil.rmtree(tmpdir)
         else:
             return {}
 
@@ -223,9 +253,10 @@ class RestoreOs(object):
         2) Download and upload to glance
         3) Create volume from glance
         4) Delete
+
         :param restore_from_timestamp:
         :type restore_from_timestamp: int
-        :param volume_id - id of attached cinder volume
+        :param volume_id: - id of attached cinder volume
         """
         (info, image) = self._create_image(volume_id, restore_from_timestamp)
         length = int(info["x-object-meta-length"])
