@@ -1,19 +1,17 @@
-"""
-Copyright 2015 Hewlett-Packard
+# Copyright 2015 Hewlett-Packard
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#     http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
 
-Licensed under the Apache License, Version 2.0 (the "License");
-you may not use this file except in compliance with the License.
-You may obtain a copy of the License at
-
-    http://www.apache.org/licenses/LICENSE-2.0
-
-Unless required by applicable law or agreed to in writing, software
-distributed under the License is distributed on an "AS IS" BASIS,
-WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-See the License for the specific language governing permissions and
-limitations under the License.
-
-"""
 
 import datetime
 import json
@@ -307,7 +305,8 @@ class Job(object):
             next_event = job_doc['job_schedule'].get('event', '')
             while next_event:
                 if next_event == Job.STOP_EVENT:
-                    LOG.info('JOB {0} event: STOP'.format(self.id))
+                    if isinstance(self.state(), StopState):
+                        LOG.info('JOB {0} event: STOP'.format(self.id))
                     next_event = self.state.stop(self, job_doc)
                 elif next_event == Job.START_EVENT:
                     LOG.info('JOB {0} event: START'.format(self.id))
@@ -336,7 +335,6 @@ class Job(object):
         freezer_action = job_action.get('freezer_action', {})
         max_retries_interval = job_action.get('max_retries_interval', 60)
         action_name = freezer_action.get('action', '')
-        config_file_name = None
         while tries:
             with tempfile.NamedTemporaryFile(delete=False) as config_file:
                 self.save_action_to_file(freezer_action, config_file)
@@ -382,8 +380,6 @@ class Job(object):
                                 ' retrying in {2} seconds'
                                 .format(self.id, action_name,
                                         max_retries_interval))
-                    # sleeping with the bloody lock, but we don't want other
-                    # actions to mess with our stuff like fs snapshots, do we ?
                     time.sleep(max_retries_interval)
             else:
                 # SUCCESS
@@ -419,46 +415,45 @@ class Job(object):
 
     def execute(self):
         result = Job.SUCCESS_RESULT
-        with self.scheduler.execution_lock:
-            with self.scheduler.lock:
-                LOG.info('job {0} running'.format(self.id))
-                self.state = RunningState
-                self.update_job_schedule_doc(status=Job.RUNNING_STATUS,
-                                             result="",
-                                             time_started=int(time.time()),
-                                             time_ended=Job.TIME_NULL)
-                self.scheduler.update_job_schedule(
-                    self.id,
-                    self.job_doc['job_schedule'])
+        with self.scheduler.lock:
+            LOG.info('job {0} running'.format(self.id))
+            self.state = RunningState
+            self.update_job_schedule_doc(status=Job.RUNNING_STATUS,
+                                         result="",
+                                         time_started=int(time.time()),
+                                         time_ended=Job.TIME_NULL)
+            self.scheduler.update_job_schedule(
+                self.id,
+                self.job_doc['job_schedule'])
 
-            self.start_session()
-            # if the job contains exec action and the scheduler passes the
-            # parameter --disable-exec job execution should fail
-            if self.contains_exec() and CONF.disable_exec:
-                LOG.info("Job {0} failed because it contains exec action "
-                         "and exec actions are disabled by scheduler"
-                         .format(self.id))
-                self.result = Job.FAIL_RESULT
-                self.finish()
-                return
-
-            for job_action in self.job_doc.get('job_actions', []):
-                if job_action.get('mandatory', False) or\
-                        (result == Job.SUCCESS_RESULT):
-
-                    action_result = self.execute_job_action(job_action)
-                    if action_result == Job.FAIL_RESULT:
-                        result = Job.FAIL_RESULT
-
-                    if action_result == Job.ABORTED_RESULT:
-                        result = Job.ABORTED_RESULT
-                else:
-                    freezer_action = job_action.get('freezer_action', {})
-                    action_name = freezer_action.get('action', '')
-                    LOG.warning("skipping {0} action".
-                                format(action_name))
-            self.result = result
+        self.start_session()
+        # if the job contains exec action and the scheduler passes the
+        # parameter --disable-exec job execution should fail
+        if self.contains_exec() and CONF.disable_exec:
+            LOG.info("Job {0} failed because it contains exec action "
+                     "and exec actions are disabled by scheduler"
+                     .format(self.id))
+            self.result = Job.FAIL_RESULT
             self.finish()
+            return
+
+        for job_action in self.job_doc.get('job_actions', []):
+            if job_action.get('mandatory', False) or\
+                    (result == Job.SUCCESS_RESULT):
+
+                action_result = self.execute_job_action(job_action)
+                if action_result == Job.FAIL_RESULT:
+                    result = Job.FAIL_RESULT
+
+                if action_result == Job.ABORTED_RESULT:
+                    result = Job.ABORTED_RESULT
+            else:
+                freezer_action = job_action.get('freezer_action', {})
+                action_name = freezer_action.get('action', '')
+                LOG.warning("skipping {0} action".
+                            format(action_name))
+        self.result = result
+        self.finish()
 
     def finish(self):
         self.update_job_schedule_doc(time_ended=int(time.time()))
