@@ -1,28 +1,26 @@
-"""
-Copyright 2015 Hewlett-Packard
+# Copyright 2015 Hewlett-Packard
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#     http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
 
-Licensed under the Apache License, Version 2.0 (the "License");
-you may not use this file except in compliance with the License.
-You may obtain a copy of the License at
-
-    http://www.apache.org/licenses/LICENSE-2.0
-
-Unless required by applicable law or agreed to in writing, software
-distributed under the License is distributed on an "AS IS" BASIS,
-WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-See the License for the specific language governing permissions and
-limitations under the License.
-
-"""
 
 import json
 import os
 import signal
 import socket
-import uuid
 
 from freezerclient import exceptions
 from oslo_log import log
+from oslo_utils import uuidutils
 import psutil
 
 
@@ -83,7 +81,8 @@ def get_jobs_from_disk(path):
         load_doc_from_json_file(f) for f in find_config_files(path)]
     for job_doc in job_doc_list:
         if job_doc:
-            job_doc['job_id'] = job_doc.get('job_id', uuid.uuid4().hex)
+            job_doc['job_id'] = job_doc.get('job_id', uuidutils.generate_uuid(
+                dashed=False))
     return [x for x in job_doc_list if x]
 
 
