@@ -16,10 +16,10 @@
 
 import abc
 import json
-import six
 import tempfile
 
 from oslo_log import log
+import six
 
 from freezer.utils import utils
 
@@ -32,6 +32,7 @@ class Storage(object):
     Any freezer storage implementation should be inherited from this abstract
     class.
     """
+    _type = None
 
     def __init__(self, skip_prepare=False):
         if not skip_prepare:
@@ -76,6 +77,10 @@ class Storage(object):
         """
         pass
 
+    @property
+    def type(self):
+        return self._type
+
     def get_latest_level_zero_increments(self, engine, hostname_backup_name,
                                          recent_to_date=None):
         """
@@ -101,7 +106,7 @@ class Storage(object):
         increments = backup.get_increments()
 
         return {level: backup for level, backup in increments.iteritems()
-                if not recent_to_date or backup.timestamp >= recent_to_date}
+                if not recent_to_date or backup.timestamp <= recent_to_date}
 
     def remove_older_than(self, engine, remove_older_timestamp,
                           hostname_backup_name):
@@ -189,14 +194,15 @@ class Backup(object):
     """
     Internal freezer representation of backup.
     Includes:
-        name (hostname_backup_name) of backup
-        timestamp of backup (when it was executed)
-        level of backup (freezer supports incremental backup)
-            Completed full backup has level 0 and can be restored without any
-            additional information.
-            Levels 1, 2, ... means that our backup is incremental and contains
-            only smart portion of information (that was actually changed
-            since the last backup)
+
+    * name (hostname_backup_name) of backup
+    * timestamp of backup (when it was executed)
+    * level of backup (freezer supports incremental backup):
+      Completed full backup has level 0 and can be restored without any
+      additional information.
+      Levels 1, 2, ... means that our backup is incremental and contains
+      only smart portion of information (that was actually changed
+      since the last backup)
     """
 
     def __init__(self, engine, hostname_backup_name,
@@ -209,9 +215,6 @@ class Backup(object):
         :type timestamp: int
         :param level: current incremental level of backup
         :type level: int
-        :param tar_meta: Is backup has or has not an attached meta
-        tar file in storage. Default = False
-        :type tar_meta: bool
         :return:
         """
         self.hostname_backup_name = hostname_backup_name
@@ -276,12 +279,17 @@ class Backup(object):
             level=int(increment[0])
         ) for increment in increments}
 
+    def _get_file(self, filename):
+        file = tempfile.NamedTemporaryFile('wb', delete=True)
+        self.storage.get_file(filename, file.name)
+        with open(file.name) as f:
+            content = f.readlines()
+        LOG.info("Content download {0}".format(content))
+        file.close()
+        return json.loads(content[0])
+
     def metadata(self):
-        metadata_file = tempfile.NamedTemporaryFile('wb', delete=True)
-        self.storage.get_file(self.metadata_path, metadata_file.name)
-        with open(metadata_file.name) as f:
-            metadata_content = f.readlines()
-        LOG.info("metadata content download {0}".format(metadata_content))
-        metadata_file.close()
-        utils.delete_file(metadata_file.name)
-        return json.loads(metadata_content[0])
+        return self._get_file(self.metadata_path)
+
+    def engine_metadata(self):
+        return self._get_file(self.engine_metadata_path)
