@@ -1,19 +1,17 @@
-"""
-(c) Copyright 2014,2015 Hewlett-Packard Development Company, L.P.
+# (c) Copyright 2014,2015 Hewlett-Packard Development Company, L.P.
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#     http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
 
-Licensed under the Apache License, Version 2.0 (the "License");
-you may not use this file except in compliance with the License.
-You may obtain a copy of the License at
-
-    http://www.apache.org/licenses/LICENSE-2.0
-
-Unless required by applicable law or agreed to in writing, software
-distributed under the License is distributed on an "AS IS" BASIS,
-WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-See the License for the specific language governing permissions and
-limitations under the License.
-
-"""
 
 import errno
 import os
@@ -33,30 +31,62 @@ class SshStorage(fslike.FsLikeStorage):
     """
     _type = 'ssh'
 
-    def __init__(self, storage_path, ssh_key_path,
-                 remote_username, remote_ip, port, max_segment_size):
+    def __init__(self, storage_path, remote_username, remote_ip, port,
+                 max_segment_size, ssh_key_path=None, remote_pwd=None):
         """
             :param storage_path: directory of storage
             :type storage_path: str
             :return:
             """
         self.ssh_key_path = ssh_key_path
+        self.remote_pwd = remote_pwd
         self.remote_username = remote_username
         self.remote_ip = remote_ip
         self.port = port
         self.ssh = None
         self.ftp = None
+        self._validate()
         self.init()
         super(SshStorage, self).__init__(
             storage_path=storage_path,
             max_segment_size=max_segment_size)
 
+    def _validate(self):
+        """
+        Validates if all parameters required to ssh are available.
+        :return: True or raises ValueError
+        """
+        if not self.remote_ip:
+            raise ValueError('Please provide --ssh-host value.')
+        elif not self.remote_username:
+            raise ValueError('Please provide --ssh-username value.')
+        elif self.remote_pwd:
+            return True
+        elif not self.ssh_key_path:
+            raise ValueError('Please provide path to ssh key using '
+                             '--ssh-key argument')
+        elif not os.path.exists(self.ssh_key_path):
+            raise ValueError('1.The {0} is required, please ensure '
+                             'it exists and ensure backup node can '
+                             'login to {1} as user {2} '
+                             'without password.\n'
+                             '2. --ssh-password argument is required and'
+                             'ensure backup node can login to {1} '
+                             'as user {2} with password.'
+                             .format(self.ssh_key_path, self.remote_ip,
+                                     self.remote_username))
+        return True
+
     def init(self):
         ssh = paramiko.SSHClient()
         ssh.load_system_host_keys()
         ssh.set_missing_host_key_policy(paramiko.AutoAddPolicy())
-        ssh.connect(self.remote_ip, username=self.remote_username,
-                    key_filename=self.ssh_key_path, port=self.port)
+        if self.remote_pwd:
+            ssh.connect(self.remote_ip, username=self.remote_username,
+                        password=self.remote_pwd, port=self.port)
+        else:
+            ssh.connect(self.remote_ip, username=self.remote_username,
+                        key_filename=self.ssh_key_path, port=self.port)
 
         # we should keep link to ssh to prevent garbage collection
         self.ssh = ssh
@@ -95,6 +125,8 @@ class SshStorage(fslike.FsLikeStorage):
             return True
 
     def get_file(self, from_path, to_path):
+        if not self.ssh.get_transport().is_alive():
+            self.init()
         self.ftp.get(from_path, to_path)
 
     def put_file(self, from_path, to_path):
